#!/usr/local/bin/perl

use strict;

##-------------------------------------------------------------

my $PASS      = 'kumasan';

my $DATAFILE  = 'log.dat';
                    # data file

my $ORDER     = 1;
                    # sort order (sort by the first parameter of @PARAMS)
                    # -1 : do not sort - the new post is always added to the end
                    #  0 : do not sort - the new post is always added to the top
                    #  1 : from small to big - ascending
                    #  2 : from big to small - descending

my $RECORDS   = 10;
                    # number of records

my @PARAMS    = qw{ score name };
                    # names of parameters
                    # The first parameter is used for sorting and MD5

my @REQPARAMS = qw{ score name };
                    # required parameters
                    # If this parameter is not defined, the script will return
                    # error message 'parameter(s) are not enough'

my @RTNPARAMS = qw{ rank message auth swfhash cgihash };
                    # names of return parameters
                    # Do not change the order if you change names of variables
                    #
                    # 1st parameter, rank
                    #       -1          : error
                    #       0           : query is out of the rank
                    #       more than 0 : rank
                    #
                    # 2nd parameter, message form cgi ( type of error, etc )
                    # 3rd parameter, authorization ( 1 - succeed : 0 - failed )
                    # 4th parameter, MD5 hash from swf
                    # 5th parameter, MD5 hash from cgi

my $USEMD5 = 1;
                    # 1 - use : 0 - no use
                    #
                    # In the case of 1, md5.pl is required, and use for authorization
                    # The script compare hashs made by MD5 algorithm
                    #
                    # ex.
                    #  - in AS,
                    #   #include "md5.as"
                    #   sendVars.pass = MD5( score.toString() + pass.toString() );
                    #
                    #  - in this script,
                    #   require 'md5.pl';
                    #   $hash = &md5::hash( $_ . $PASS );
                    #
                    # In the case of 0, md5.pl is not required
                    # The script compare passwords
                    #
                    # ex.
                    #  - in AS,
                    #  sendVars.pass = pass;

my @MESSAGES;
$MESSAGES[0] = [     0, 'out of rank' ];
$MESSAGES[1] = [ undef, 'rank in' ];
$MESSAGES[2] = [ undef, 'rank check mode' ];
$MESSAGES[3] = [    -1, 'failed to open log file' ];
$MESSAGES[4] = [    -1, 'bad password/hash' ];
$MESSAGES[5] = [    -1, 'parameter(s) are not enough' ];

##-------------------------------------------------------------
#use CGI::Carp qw(fatalsToBrowser);
$USEMD5 and require 'md5.pl';


#
#  GLOBAL VARIABLES
#

my %form;
my $authorized;
my $is_check;
my $hash;
my $rank;
my $buffer;


#
# DECODE QUERY
#

if( $ENV{'REQUEST_METHOD'} eq "POST" ) {
	$ENV{'CONTENT_LENGTH'} > 1024 * 4 and die;
	read( STDIN, $buffer, $ENV{'CONTENT_LENGTH'} );
} elsif ( $ENV{'REQUEST_METHOD'} eq "GET" ) {
	$buffer = $ENV{'QUERY_STRING'};
}
foreach ( split( /[&;]/,  $buffer ) ) {
	my ( $key, $value ) = split( /=/, $_ );
	$form{$key} = $value if grep( /^$key$/, ( @PARAMS, 'pass', 'mode' ) );
}
$is_check = ( $form{'mode'} eq 'check' )? 1 : 0;


#
# AUTHENTICATION
#

if ( $PASS ne undef and $USEMD5 ) {
	$_ = $form{$PARAMS[0]};
	$_ =~ s/%([0-9A-Fa-f][0-9A-Fa-f])/pack('H2', $1)/eg;
	$hash = &md5::hash( $_ . $PASS );
	$authorized = ( $hash eq $form{'pass'} )? 1 : 0;
	$authorized or &output(4) unless $is_check;
} elsif ( $PASS ne undef ) {
	$authorized = ( $PASS eq $form{'pass'} )? 1 : 0;
	$hash = 'MD5+is+not+used';
	$authorized or &output(4) unless $is_check;
} else {
	$authorized = 1;
}


#
# CHECK REQUIRED PARAMETERS
#

unless ( $is_check ) {
	foreach ( @REQPARAMS ) {
		$form{$_} eq undef and &output(5);
	}
}


#
# DATA SORTING
#

open( SCORE, "+<$DATAFILE" ) or &output(3);
flock( SCORE, 2 ) or &output(3);
my $buf = <SCORE>;

my @records;
foreach my $param ( split(/&/, $buf) ) {
	foreach ( 0 .. @PARAMS - 1 ) {
		$records[ $1 - 1 ]->[$_] = $2 if $param =~ /^$PARAMS[$_](\d+)\=(.*?)$/;
	}
}
my $newrecord = [];
foreach my $key ( keys(%form) ) {
	foreach ( 0 .. @PARAMS - 1 ) {
		$newrecord->[$_] =  $form{$key} if $key eq $PARAMS[$_];
	}
}
$newrecord->[$_ = @PARAMS] = 1;

my @result;
# -1 : do not sort - the new post is always added to the end
if ( $ORDER == -1 ) {
	push( @records, $newrecord );
	my $end = @records - 1;
	my $top = ( @records - $RECORDS )? @records - $RECORDS : 0;
	@result = @records[ $top .. $end ];

# 0 : do not sort - the new post is always added to the top
} elsif ( $ORDER == 0 ) {
	@result = ( $newrecord, @records );

# 1 : from small to big - ascending
} elsif ( $ORDER == 1 ) {
	push( @records, $newrecord );
	@result = sort { $a->[0] <=> $b->[0] } @records;

# 2 : from big to small - descending
} elsif ( $ORDER == 2 ) {
	push( @records, $newrecord );
	@result = sort { $b->[0] <=> $a->[0] } @records;
}

$rank = &check_rank( \@result );
if ( $is_check ) {
	close SCORE;
	&output(2);
} elsif ( $rank ) {
	seek( SCORE, 0, 0 );
	foreach my $num ( 1 .. $RECORDS ) {
		next if $result[$num - 1] eq undef;
		foreach ( 0 .. @PARAMS - 1 ) {
			next if $result[$num - 1]->[$_] eq undef;
			print SCORE "$PARAMS[$_]$num=$result[$num - 1]->[$_]";
			print SCORE '&' unless ( $_ == @PARAMS - 1 || $_ == @{$result[$num - 1]} - 1 and $num == $RECORDS || $num == @result );
		}
	}
	truncate( SCORE, tell );
	close SCORE;
	&output(1);
} else {
	close SCORE;
	&output(0);
}

exit;

##-------------------------------------------------------------

sub check_rank {
	my $result = shift;
	my $rank;

	foreach $rank ( 1 .. $RECORDS ) {
		next if ${$result}[$rank - 1] eq undef;
		${$result}[$rank - 1]->[$_ = @PARAMS] and return $rank;
	}
	return 0;
}

sub output {
	my $code = shift;
	print "Content-type: text/plain;\n\n";
	$MESSAGES[$code]->[0] = $rank if $MESSAGES[$code]->[0] eq undef;
	$form{'pass'} = '********' unless $USEMD5;
	&url_enc( \$MESSAGES[$code]->[1] );
	print	"$RTNPARAMS[0]=$MESSAGES[$code]->[0]&",
			"$RTNPARAMS[1]=$MESSAGES[$code]->[1]&",
			"$RTNPARAMS[2]=$authorized&",
			"$RTNPARAMS[3]=$form{'pass'}&",
			"$RTNPARAMS[4]=$hash";
	exit;
}

sub url_enc {
	my $p = shift;
	$$p =~ s/([^\w ])/'%' . unpack('H2', $1)/eg;
	$$p =~ tr/ /+/;
}

